{-# LANGUAGE TemplateHaskell, CPP, MagicHash, TypeSynonymInstances #-}

{- |
  Module      :  Language.Haskell.TH.Instances.Lift
  Copyright   :  (c) Matt Morrow 2008
  License     :  BSD3
  Maintainer  :  Matt Morrow <mjm2002@gmail.com>
  Stability   :  experimental
  Portability :  portable (template-haskell)
-}

module Language.Haskell.TH.Instances.Lift () where

import Language.Haskell.TH.Syntax.Internals
import Language.Haskell.TH.Syntax
import Language.Haskell.TH.Lib
import Language.Haskell.TH.Ppr
import Data.List(intercalate)
import GHC.Prim
import GHC.Exts

deriving instance Ord Exp
deriving instance Ord Dec
deriving instance Ord Stmt
deriving instance Ord Type
deriving instance Ord Foreign
deriving instance Ord FunDep
deriving instance Ord Con
deriving instance Ord Body
deriving instance Ord Clause
deriving instance Ord Strict
deriving instance Ord Safety
deriving instance Ord Callconv
deriving instance Ord Guard
deriving instance Ord Range
deriving instance Ord Match
deriving instance Ord Pat
deriving instance Ord Lit
deriving instance Ord Kind
deriving instance Ord FamFlavour
deriving instance Ord Pragma
deriving instance Ord TyVarBndr
deriving instance Ord Pred
deriving instance Ord InlineSpec



instance Lift () where
  lift () = [|()|]

instance Show Loc where
  show (Loc f p m s e) =
    intercalate " " $
      ("Loc":fmap show [f,p,m]++[show s,show e])

instance Eq Loc where
  (Loc a b c d e) == (Loc v w x y z)
    = and $ [d==y,e==z] ++
        (zipWith (==) [a,b,c] [v,w,x])

-- TODO: make this better
instance Ppr Loc where
  ppr = showtextl . show


instance Ppr Lit where
  ppr l = ppr (LitE l)

-- comic relief from HERA
instance Lift Rational where lift _ = error "Rational.. what are you doing!"

instance Lift Name where
  lift (Name occName nameFlavour) = [| Name occName nameFlavour |]



instance Lift ModName where
  lift (ModName s) = [|ModName s|]
instance Lift PkgName where
  lift (PkgName s) = [|PkgName s|]
instance Lift OccName where
  lift (OccName s) = [|OccName s|]




instance Lift Pragma where
  lift (InlineP a b) = [|InlineP a b|]
  lift (SpecialiseP a b c) = [|SpecialiseP a b c|]


instance Lift InlineSpec where
  lift (InlineSpec a b c) = [|InlineSpec a b c|]

instance Lift Pred where
  lift (ClassP a b) = [|ClassP a b|]
  lift (EqualP a b) = [|EqualP a b|]




instance Lift TyVarBndr where
  lift (PlainTV a) = [|PlainTV a|]
  lift (KindedTV a b) = [|KindedTV a b|]

instance Lift Kind where
  lift StarK = [|StarK|]
  lift (ArrowK a b) = [|ArrowK a b|]



instance Lift FamFlavour where
  lift TypeFam = [|TypeFam|]
  lift DataFam = [|DataFam|]



instance Lift NameFlavour where
  lift NameS = [| NameS |]
  lift (NameQ modName) = [| NameQ modName |]
  lift (NameU i) = [| case $( lift (I# i) ) of
                          I# i' -> NameU i' |]
  lift (NameL i) = [| case $( lift (I# i) ) of
                          I# i' -> NameL i' |]
  lift (NameG nameSpace pkgName modName)
    = [| NameG nameSpace pkgName modName |]

instance Lift NameSpace where
  lift VarName = [| VarName |]
  lift DataName = [| DataName |]
  lift TcClsName = [| TcClsName |]


instance Lift Dec where
        lift (FunD x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "FunD")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (ValD x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "ValD")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (DataD x0 x1 x2 x3 x4)
          = appE
              (appE
                 (appE
                    (appE
                       (appE
                          (conE
                             (Name (OccName "DataD")
                                (NameG DataName (PkgName "template-haskell")
                                   (ModName "Language.Haskell.TH.Syntax"))))
                          (lift x0))
                       (lift x1))
                    (lift x2))
                 (lift x3))
              (lift x4)
        lift (NewtypeD x0 x1 x2 x3 x4)
          = appE
              (appE
                 (appE
                    (appE
                       (appE
                          (conE
                             (Name (OccName "NewtypeD")
                                (NameG DataName (PkgName "template-haskell")
                                   (ModName "Language.Haskell.TH.Syntax"))))
                          (lift x0))
                       (lift x1))
                    (lift x2))
                 (lift x3))
              (lift x4)
        lift (TySynD x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "TySynD")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (ClassD x0 x1 x2 x3 x4)
          = appE
              (appE
                 (appE
                    (appE
                       (appE
                          (conE
                             (Name (OccName "ClassD")
                                (NameG DataName (PkgName "template-haskell")
                                   (ModName "Language.Haskell.TH.Syntax"))))
                          (lift x0))
                       (lift x1))
                    (lift x2))
                 (lift x3))
              (lift x4)
        lift (InstanceD x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "InstanceD")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (SigD x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "SigD")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (ForeignD x0)
          = appE
              (conE
                 (Name (OccName "ForeignD")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (PragmaD x0)
          = appE
              (conE
                 (Name (OccName "PragmaD")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (FamilyD x0 x1 x2 x3)
          = appE
                (appE
                  (appE
                      (appE
                        (conE
                            (Name (OccName "FamilyD")
                              (NameG DataName (PkgName "template-haskell")
                                  (ModName "Language.Haskell.TH.Syntax"))))
                        (lift x0))
                      (lift x1))
                  (lift x2))
                (lift x3)
        lift (DataInstD x0 x1 x2 x3 x4)
          = appE
              (appE
                 (appE
                    (appE
                       (appE
                          (conE
                             (Name (OccName "DataInstD")
                                (NameG DataName (PkgName "template-haskell")
                                   (ModName "Language.Haskell.TH.Syntax"))))
                          (lift x0))
                       (lift x1))
                    (lift x2))
                 (lift x3))
              (lift x4)
        lift (NewtypeInstD x0 x1 x2 x3 x4)
          = appE
              (appE
                 (appE
                    (appE
                       (appE
                          (conE
                             (Name (OccName "NewtypeInstD")
                                (NameG DataName (PkgName "template-haskell")
                                   (ModName "Language.Haskell.TH.Syntax"))))
                          (lift x0))
                       (lift x1))
                    (lift x2))
                 (lift x3))
              (lift x4)
        lift (TySynInstD x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "TySynInstD")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)






instance Lift Exp where
        lift (VarE x0)
          = appE
              (conE
                 (Name (OccName "VarE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ConE x0)
          = appE
              (conE
                 (Name (OccName "ConE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (LitE x0)
          = appE
              (conE
                 (Name (OccName "LitE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (AppE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "AppE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (InfixE x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "InfixE")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (LamE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "LamE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (TupE x0)
          = appE
              (conE
                 (Name (OccName "TupE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (CondE x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "CondE")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (LetE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "LetE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (CaseE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "CaseE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (DoE x0)
          = appE
              (conE
                 (Name (OccName "DoE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (CompE x0)
          = appE
              (conE
                 (Name (OccName "CompE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ArithSeqE x0)
          = appE
              (conE
                 (Name (OccName "ArithSeqE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ListE x0)
          = appE
              (conE
                 (Name (OccName "ListE")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (SigE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "SigE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (RecConE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "RecConE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (RecUpdE x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "RecUpdE")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)

instance Lift Lit where
        lift (CharL x0)
          = appE
              (conE
                 (Name (OccName "CharL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (StringL x0)
          = appE
              (conE
                 (Name (OccName "StringL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (IntegerL x0)
          = appE
              (conE
                 (Name (OccName "IntegerL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (RationalL x0)
          = appE
              (conE
                 (Name (OccName "RationalL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (IntPrimL x0)
          = appE
              (conE
                 (Name (OccName "IntPrimL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 609
        lift (WordPrimL x0)
          = appE
              (conE
                 (Name (OccName "WordPrimL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
#endif
        lift (FloatPrimL x0)
          = appE
              (conE
                 (Name (OccName "FloatPrimL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (DoublePrimL x0)
          = appE
              (conE
                 (Name (OccName "DoublePrimL")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)

instance Lift Pat where
        lift (LitP x0)
          = appE
              (conE
                 (Name (OccName "LitP")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (VarP x0)
          = appE
              (conE
                 (Name (OccName "VarP")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (TupP x0)
          = appE
              (conE
                 (Name (OccName "TupP")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ConP x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "ConP")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (InfixP x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "InfixP")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (TildeP x0)
          = appE
              (conE
                 (Name (OccName "TildeP")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (BangP x0)
          = appE
              (conE
                 (Name (OccName "BangP")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (AsP x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "AsP")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (WildP)
          = conE
              (Name (OccName "WildP")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (RecP x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "RecP")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (ListP x0)
          = appE
              (conE
                 (Name (OccName "ListP")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (SigP x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "SigP")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)

instance Lift Body where
        lift (GuardedB x0)
          = appE
              (conE
                 (Name (OccName "GuardedB")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (NormalB x0)
          = appE
              (conE
                 (Name (OccName "NormalB")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)

instance Lift Con where
        lift (NormalC x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "NormalC")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (RecC x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "RecC")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (InfixC x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "InfixC")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (ForallC x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "ForallC")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)

instance Lift Clause where
        lift (Clause x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "Clause")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)

instance Lift Guard where
        lift (NormalG x0)
          = appE
              (conE
                 (Name (OccName "NormalG")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (PatG x0)
          = appE
              (conE
                 (Name (OccName "PatG")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)

instance Lift Strict where
        lift (IsStrict)
          = conE
              (Name (OccName "IsStrict")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (NotStrict)
          = conE
              (Name (OccName "NotStrict")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))

instance Lift FunDep where
        lift (FunDep x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "FunDep")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)

instance Lift Foreign where
        lift (ImportF x0 x1 x2 x3 x4)
          = appE
              (appE
                 (appE
                    (appE
                       (appE
                          (conE
                             (Name (OccName "ImportF")
                                (NameG DataName (PkgName "template-haskell")
                                   (ModName "Language.Haskell.TH.Syntax"))))
                          (lift x0))
                       (lift x1))
                    (lift x2))
                 (lift x3))
              (lift x4)
        lift (ExportF x0 x1 x2 x3)
          = appE
              (appE
                 (appE
                    (appE
                       (conE
                          (Name (OccName "ExportF")
                             (NameG DataName (PkgName "template-haskell")
                                (ModName "Language.Haskell.TH.Syntax"))))
                       (lift x0))
                    (lift x1))
                 (lift x2))
              (lift x3)

instance Lift Callconv where
        lift (CCall)
          = conE
              (Name (OccName "CCall")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (StdCall)
          = conE
              (Name (OccName "StdCall")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))

instance Lift Safety where
        lift (Unsafe)
          = conE
              (Name (OccName "Unsafe")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (Safe)
          = conE
              (Name (OccName "Safe")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (Threadsafe)
          = conE
              (Name (OccName "Threadsafe")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))

instance Lift Match where
        lift (Match x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "Match")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)

instance Lift Stmt where
        lift (BindS x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "BindS")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (LetS x0)
          = appE
              (conE
                 (Name (OccName "LetS")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (NoBindS x0)
          = appE
              (conE
                 (Name (OccName "NoBindS")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ParS x0)
          = appE
              (conE
                 (Name (OccName "ParS")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)

instance Lift Range where
        lift (FromR x0)
          = appE
              (conE
                 (Name (OccName "FromR")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (FromThenR x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "FromThenR")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (FromToR x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "FromToR")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (FromThenToR x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "FromThenToR")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)

instance Lift Type where
        lift (ForallT x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "ForallT")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (VarT x0)
          = appE
              (conE
                 (Name (OccName "VarT")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ConT x0)
          = appE
              (conE
                 (Name (OccName "ConT")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (TupleT x0)
          = appE
              (conE
                 (Name (OccName "TupleT")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ArrowT)
          = conE
              (Name (OccName "ArrowT")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (ListT)
          = conE
              (Name (OccName "ListT")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (AppT x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "AppT")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)
        lift (SigT x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "SigT")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)






instance Lift Info where
        lift (ClassI x0)
          = appE
              (conE
                 (Name (OccName "ClassI")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (ClassOpI x0 x1 x2 x3)
          = appE
              (appE
                 (appE
                    (appE
                       (conE
                          (Name (OccName "ClassOpI")
                             (NameG DataName (PkgName "template-haskell")
                                (ModName "Language.Haskell.TH.Syntax"))))
                       (lift x0))
                    (lift x1))
                 (lift x2))
              (lift x3)
        lift (TyConI x0)
          = appE
              (conE
                 (Name (OccName "TyConI")
                    (NameG DataName (PkgName "template-haskell")
                       (ModName "Language.Haskell.TH.Syntax"))))
              (lift x0)
        lift (PrimTyConI x0 x1 x2)
          = appE
              (appE
                 (appE
                    (conE
                       (Name (OccName "PrimTyConI")
                          (NameG DataName (PkgName "template-haskell")
                             (ModName "Language.Haskell.TH.Syntax"))))
                    (lift x0))
                 (lift x1))
              (lift x2)
        lift (DataConI x0 x1 x2 x3)
          = appE
              (appE
                 (appE
                    (appE
                       (conE
                          (Name (OccName "DataConI")
                             (NameG DataName (PkgName "template-haskell")
                                (ModName "Language.Haskell.TH.Syntax"))))
                       (lift x0))
                    (lift x1))
                 (lift x2))
              (lift x3)
        lift (VarI x0 x1 x2 x3)
          = appE
              (appE
                 (appE
                    (appE
                       (conE
                          (Name (OccName "VarI")
                             (NameG DataName (PkgName "template-haskell")
                                (ModName "Language.Haskell.TH.Syntax"))))
                       (lift x0))
                    (lift x1))
                 (lift x2))
              (lift x3)
        lift (TyVarI x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "TyVarI")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)

instance Lift Fixity where
        lift (Fixity x0 x1)
          = appE
              (appE
                 (conE
                    (Name (OccName "Fixity")
                       (NameG DataName (PkgName "template-haskell")
                          (ModName "Language.Haskell.TH.Syntax"))))
                 (lift x0))
              (lift x1)

instance Lift FixityDirection where
        lift (InfixL)
          = conE
              (Name (OccName "InfixL")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (InfixR)
          = conE
              (Name (OccName "InfixR")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
        lift (InfixN)
          = conE
              (Name (OccName "InfixN")
                 (NameG DataName (PkgName "template-haskell")
                    (ModName "Language.Haskell.TH.Syntax")))
